==> libevnet-0.3.0.tgz: libevnet-0.3.0/regress/google-tls.c <==
#include <stdio.h>
#include <stdlib.h>

#include <string.h>

#include <assert.h>

#include <err.h>

#include <sys/queue.h>
#include <netdb.h>

#include <event.h>

#include "socket.h"
#include "bufio.h"
#include "lookup.h"
#include "tls.h"


static enum bufio_action main_gets_catch(struct bufio *io, const char *line, size_t len, enum bufio_errno err, void *fd) {
	if (!(err == BUFIO_ESUCCESS || err == BUFIO_ENOMORE))
		errx(EXIT_FAILURE, "bufio_gets: %s", bufio_strerror(io));

	if (len > 0)
		warnx("read %d bytes: %.*s", (int)len, (int)(len - 1), line);

	if (err == BUFIO_ENOMORE) {
		bufio_close(io);

		warnx("got EOF");

		return BUFIO_COMPLETE;
	}

	return BUFIO_RESTART;
} /* main_gets_catch() */


static enum bufio_action main_write_catch(struct bufio *io, const void *request, size_t len, enum bufio_errno err, void *fd) {
	if (err != BUFIO_ESUCCESS)
		errx(EXIT_FAILURE, "bufio_write: %s", bufio_strerror(io));

	if (len != strlen(request)) {
		assert(0 == bufio_write(io, ((char *)request) + len, strlen(((char *)request) + len), &main_write_catch, io, &(struct timeval){ 2, 0 }));

		return 0;	
	}

	assert(0 == bufio_gets(io, &main_gets_catch, io, &(struct timeval){ 2, 0 }));

	return 0;
} /* main_write_catch() */


static void main_secure_catch(struct tls *tls, enum tls_errno err, void *fd) {
	static const char request[]	= "GET / HTTP/1.0\n\n";
	struct bufio *io;

	assert(err == TLS_ESUCCESS);

	warnx("caught TLS handshake with google.com");

	assert(io = bufio_open((int)fd, 0, 0, 0));

	bufio_tls_start(io, tls);

	assert(0 == bufio_write(io, request, sizeof request, &main_write_catch, io, &(struct timeval){ 2, 0 }));

	return /* void */;
} /* main_secure_catch() */


static void main_connect_catch(int fd, struct sockaddr *sa, socklen_t salen, enum socket_errno err, void *arg) {
	static const char request[]	= "GET / HTTP/1.0\n\n";
	static struct tls_context *tlsctx;
	struct tls *tls;

	assert(err == SOCKET_ESUCCESS);

	warnx("caught connect to google.com");

	socket_close(arg);

	assert(tlsctx = tls_context_open(0, 0, 0));
	assert(tls = tls_open(fd, tlsctx, 0, 0, 0));

	tls_connect(tls, &main_secure_catch, (void *)fd, 0);

	return /* void */;
} /* main_connect_catch() */


int main(void) {
	struct socket_service *s;
	struct socket_name soname;

	event_init();
	lookup_init(0, 0, 0);
	tls_init();

	if (0 == (s = socket_open(0, 0, 0)))
		err(EXIT_FAILURE, "socket_open");

	soname_set_host(&soname, "google.com", "https");

	socket_connect(0, s, &soname, &main_connect_catch, s, 0);

	event_loop(0);

	return 0;
}

==> libevnet-0.3.1.tgz: libevnet-0.3.1/regress/google-tls.c <==
#include <stdio.h>
#include <stdlib.h>

#include <inttypes.h>

#include <string.h>

#include <assert.h>

#include <err.h>

#include <sys/queue.h>
#include <netdb.h>

#include <event.h>

#include "socket.h"
#include "bufio.h"
#include "lookup.h"
#include "tls.h"


static enum bufio_action main_gets_catch(struct bufio *io, const char *line, size_t len, enum bufio_errno err, void *fd) {
	if (!(err == BUFIO_ESUCCESS || err == BUFIO_ENOMORE))
		errx(EXIT_FAILURE, "bufio_gets: %s", bufio_strerror(io));

	if (len > 0)
		warnx("read %d bytes: %.*s", (int)len, (int)(len - 1), line);

	if (err == BUFIO_ENOMORE) {
		bufio_close(io);

		warnx("got EOF");

		return BUFIO_COMPLETE;
	}

	return BUFIO_RESTART;
} /* main_gets_catch() */


static enum bufio_action main_write_catch(struct bufio *io, const void *request, size_t len, enum bufio_errno err, void *fd) {
	if (err != BUFIO_ESUCCESS)
		errx(EXIT_FAILURE, "bufio_write: %s", bufio_strerror(io));

	if (len != strlen(request)) {
		assert(0 == bufio_write(io, ((char *)request) + len, strlen(((char *)request) + len), &main_write_catch, io, &(struct timeval){ 2, 0 }));

		return 0;	
	}

	assert(0 == bufio_gets(io, &main_gets_catch, io, &(struct timeval){ 2, 0 }));

	return 0;
} /* main_write_catch() */


static void main_secure_catch(struct tls *tls, enum tls_errno err, void *fd) {
	static const char request[]	= "GET / HTTP/1.0\n\n";
	struct bufio *io;

	assert(err == TLS_ESUCCESS);

	warnx("caught TLS handshake with google.com");

	assert(io = bufio_open((intptr_t)fd, 0, 0, 0));

	bufio_tls_start(io, tls);

	assert(0 == bufio_write(io, request, sizeof request - 1, &main_write_catch, io, &(struct timeval){ 2, 0 }));

	return /* void */;
} /* main_secure_catch() */


static void main_connect_catch(int fd, struct connect_info *i, enum socket_errno err, void *arg) {
	static const char request[]	= "GET / HTTP/1.0\n\n";
	static struct tls_context *tlsctx;
	struct tls *tls;

	assert(err == SOCKET_ESUCCESS);

	warnx("caught connect to google.com");

	socket_close(arg);

	assert(tlsctx = tls_context_open(0, 0, 0));
	assert(tls = tls_open(fd, tlsctx, 0, 0, 0));

	tls_connect(tls, &main_secure_catch, (void *)(intptr_t)fd, 0);

	return /* void */;
} /* main_connect_catch() */


int main(void) {
	struct socket *s;
	struct socket_name soname;

	event_init();
	lookup_init(0, 0, 0);
	tls_init();

	if (0 == (s = socket_open(0, 0, 0)))
		err(EXIT_FAILURE, "socket_open");

	soname_set_host(&soname, "google.com", "https");

	socket_connect(s, &soname, &main_connect_catch, s, 0);

	event_loop(0);

	return 0;
}
